@@ -27,6 +27,7 @@
 // Mime types of known filters
 static const char application_bzip[] = "application/x-bzip";
 static const char application_lzma[] = "application/x-lzma";
+static const char application_lzip[] = "application/x-lzip";
 static const char application_xz[] = "application/x-xz";
 static const char application_zstd[] = "application/zstd";
 
@@ -60,7 +61,7 @@ public:
     bool fillTempFile(const QString &fileName);
     bool writeBackTempFile(const QString &fileName);
     void fillBuffer(char *buffer, const char *mode, qint64 size, const QDateTime &mtime, char typeflag, const char *uname, const char *gname);
-    void writeLonglink(char *buffer, const QByteArray &name, char typeflag, const char *uname, const char *gname);
+    [[nodiscard]] bool writeLonglink(char *buffer, const QByteArray &name, char typeflag, const char *uname, const char *gname);
     qint64 readRawHeader(char *buffer);
     bool readLonglink(char *buffer, QByteArray &longlink);
     qint64 readHeader(char *buffer, QString &name, QString &symlink);
@@ -122,6 +123,9 @@ bool KTar::createDevice(QIODevice::OpenMode mode)
         } else if (mime.inherits(QStringLiteral("application/x-lzma-compressed-tar")) || mime.inherits(QString::fromLatin1(application_lzma))) {
             // lzma compressed tar file (with possibly invalid file name), ask for xz filter
             d->mimetype = QString::fromLatin1(application_lzma);
+        } else if (mime.inherits(QStringLiteral("application/x-lzip-compressed-tar")) || mime.inherits(QString::fromLatin1(application_lzip))) {
+            // lzip compressed tar file (with possibly invalid file name), ask for lzip filter
+            d->mimetype = QString::fromLatin1(application_lzip);
         } else if (mime.inherits(QStringLiteral("application/x-xz-compressed-tar")) || mime.inherits(QString::fromLatin1(application_xz))) {
             // xz compressed tar file (with possibly invalid name), ask for xz filter
             d->mimetype = QString::fromLatin1(application_xz);
@@ -159,7 +163,9 @@ bool KTar::createDevice(QIODevice::OpenMode mode)
         Q_ASSERT(!d->tmpFile);
         d->tmpFile = new QTemporaryFile();
         d->tmpFile->setFileTemplate(QDir::tempPath() + QLatin1Char('/') + QLatin1String("ktar-XXXXXX.tar"));
-        d->tmpFile->open();
+        if (!d->tmpFile->open()) {
+            return false;
+        }
         // qCDebug(KArchiveLog) << "creating tempfile:" << d->tmpFile->fileName();
 
         setDevice(d->tmpFile);
@@ -243,8 +249,7 @@ bool KTar::KTarPrivate::readLonglink(char *buffer, QByteArray &longlink)
     qint64 size = QByteArray(buffer + 0x7c, 12).trimmed().toLongLong(nullptr, 8 /*octal*/);
 
     size--; // ignore trailing null
-    if (size > std::numeric_limits<int>::max() - 32) { // QByteArray can't really be INT_MAX big, it's max size is something between INT_MAX - 32 and INT_MAX
-                                                       // depending the platform so just be safe
+    if (size > kMaxQByteArraySize) {
         qCWarning(KArchiveLog) << "Failed to allocate memory for longlink of size" << size;
         return false;
     }
@@ -529,15 +534,15 @@ bool KTar::openArchive(QIODevice::OpenMode mode)
                         delete e;
                     }
                 } else {
-                    rootDir()->addEntry(e);
+                    // We don't want to fail opening potentially malformed files, so void the return value
+                    (void)rootDir()->addEntryV2(e);
                 }
             } else {
                 // In some tar files we can find dir/./file => call cleanPath
                 QString path = QDir::cleanPath(name.left(pos));
                 // Ensure container directory exists, create otherwise
-                KArchiveDirectory *d = findOrCreate(path);
-                if (d) {
-                    d->addEntry(e);
+                if (KArchiveDirectory *dir = findOrCreate(path); dir != nullptr) {
+                    (void)dir->addEntryV2(e);
                 } else {
                     delete e;
                     return false;
@@ -570,6 +575,7 @@ bool KTar::KTarPrivate::writeBackTempFile(const QString &fileName)
     if (MimeType::application_gzip_old() == mimetype ||
         QLatin1String(application_bzip) == mimetype ||
         QLatin1String(application_lzma) == mimetype ||
+        QLatin1String(application_lzip) == mimetype ||
         QLatin1String(application_xz) == mimetype) {
         /* clang-format on */
         forced = true;
@@ -595,7 +601,12 @@ bool KTar::KTarPrivate::writeBackTempFile(const QString &fileName)
     qint64 len;
     while (!file->atEnd()) {
         len = file->read(buffer.data(), buffer.size());
-        dev.write(buffer.data(), len); // TODO error checking
+        if (dev.write(buffer.data(), len) != len) {
+            file->close();
+            dev.close();
+            q->setErrorString(tr("Failed to write back temp file: %1").arg(dev.errorString()));
+            return false;
+        }
     }
     file->close();
     dev.close();
@@ -647,7 +658,7 @@ bool KTar::doFinishWriting(qint64 size)
     return true;
 }
 
-/*** Some help from the tar sources
+/*!* Some help from the tar sources
 struct posix_header
 {                               byte offset
   char name[100];               *   0 *     0x0
@@ -734,22 +745,32 @@ void KTar::KTarPrivate::fillBuffer(char *buffer, const char *mode, qint64 size,
     memcpy(buffer + 0x94, s.constData(), 6);
 }
 
-void KTar::KTarPrivate::writeLonglink(char *buffer, const QByteArray &name, char typeflag, const char *uname, const char *gname)
+bool KTar::KTarPrivate::writeLonglink(char *buffer, const QByteArray &name, char typeflag, const char *uname, const char *gname)
 {
     strcpy(buffer, "././@LongLink");
     qint64 namelen = name.length() + 1;
     fillBuffer(buffer, "     0", namelen, QDateTime(), typeflag, uname, gname);
-    q->device()->write(buffer, 0x200); // TODO error checking
+
+    if (q->device()->write(buffer, 0x200) != 0x200) {
+        q->setErrorString(tr("Couldn't write long link: %1").arg(q->device()->errorString()));
+        return false;
+    }
+
     qint64 offset = 0;
     while (namelen > 0) {
         int chunksize = qMin(namelen, 0x200LL);
         memcpy(buffer, name.data() + offset, chunksize);
         // write long name
-        q->device()->write(buffer, 0x200); // TODO error checking
+        if (q->device()->write(buffer, 0x200) != 0x200) {
+            q->setErrorString(tr("Couldn't write long link: %1").arg(q->device()->errorString()));
+            return false;
+        }
         // not even needed to reclear the buffer, tar doesn't do it
         namelen -= chunksize;
         offset += 0x200;
     } /*wend*/
+
+    return true;
 }
 
 bool KTar::doPrepareWriting(const QString &name,
@@ -813,8 +834,8 @@ bool KTar::doPrepareWriting(const QString &name,
     const QByteArray gname = group.toLocal8Bit();
 
     // If more than 100 bytes, we need to use the LongLink trick
-    if (encodedFileName.length() > 99) {
-        d->writeLonglink(buffer, encodedFileName, 'L', uname.constData(), gname.constData());
+    if (encodedFileName.length() > 99 && !d->writeLonglink(buffer, encodedFileName, 'L', uname.constData(), gname.constData())) {
+        return false;
     }
 
     // Write (potentially truncated) name
@@ -880,8 +901,8 @@ bool KTar::doWriteDir(const QString &name,
     QByteArray gname = group.toLocal8Bit();
 
     // If more than 100 bytes, we need to use the LongLink trick
-    if (encodedDirname.length() > 99) {
-        d->writeLonglink(buffer, encodedDirname, 'L', uname.constData(), gname.constData());
+    if (encodedDirname.length() > 99 && !d->writeLonglink(buffer, encodedDirname, 'L', uname.constData(), gname.constData())) {
+        return false;
     }
 
     // Write (potentially truncated) name
@@ -941,11 +962,11 @@ bool KTar::doWriteSymLink(const QString &name,
     QByteArray gname = group.toLocal8Bit();
 
     // If more than 100 bytes, we need to use the LongLink trick
-    if (encodedTarget.length() > 99) {
-        d->writeLonglink(buffer, encodedTarget, 'K', uname.constData(), gname.constData());
+    if (encodedTarget.length() > 99 && !d->writeLonglink(buffer, encodedTarget, 'K', uname.constData(), gname.constData())) {
+        return false;
     }
-    if (encodedFileName.length() > 99) {
-        d->writeLonglink(buffer, encodedFileName, 'L', uname.constData(), gname.constData());
+    if (encodedFileName.length() > 99 && !d->writeLonglink(buffer, encodedFileName, 'L', uname.constData(), gname.constData())) {
+        return false;
     }
 
     // Write (potentially truncated) name
